Package: multiWGCNA
Type: Package
Title: multiWGCNA
Version: 1.10.0
Authors@R: c(
    person("Dario", "Tommasini", email="dtommasini0@gmail.com",
      role = c("aut","cre"),
      comment = c(ORCID = "0000-0002-1214-6547")),
    person("Brent", "Fogel", role = c("aut","ctb")))
Description: An R package for deeping mining gene co-expression networks in
    multi-trait expression data. Provides functions for analyzing, comparing,
    and visualizing WGCNA networks across conditions. multiWGCNA was designed
    to handle the common case where there are multiple biologically meaningful
    sample traits, such as disease vs wildtype across development or anatomical
    region.
License: GPL-3
Encoding: UTF-8
LazyData: true
Suggests: BiocStyle, doParallel, ExperimentHub, knitr, markdown,
        rmarkdown, testthat (>= 3.0.0), vegan
VignetteBuilder: knitr
RoxygenNote: 7.3.2
biocViews: Sequencing, RNASeq, GeneExpression, DifferentialExpression,
        Regression, Clustering
Imports: stringr, readr, WGCNA, magrittr, dplyr, reshape2, data.table,
        patchwork, scales, igraph, flashClust, ggplot2, dcanr, cowplot,
        ggrepel, methods, SummarizedExperiment, ggraph, tidyr
Depends: R (>= 4.3.0), ggalluvial
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/multiWGCNA
git_branch: RELEASE_3_23
git_last_commit: 3076bc8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:34:39 UTC; biocbuild
Author: Dario Tommasini [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1214-6547>),
  Brent Fogel [aut, ctb]
Maintainer: Dario Tommasini <dtommasini0@gmail.com>
