# Generated by roxygen2: do not edit by hand

export(GetDatExpr)
export(PreservationPermutationTest)
export(PreservationScoreDistribution)
export(TOMFlowPlot)
export(bidirectionalBestMatches)
export(cleanDatExpr)
export(coexpressionLineGraph)
export(computeOverlapsFromWGCNA)
export(constructNetworks)
export(diffCoexpression)
export(diffModuleExpression)
export(drawMultiWGCNAnetwork)
export(getPreservation)
export(iterate)
export(makeTraitTable)
export(moduleComparisonPlot)
export(moduleExpressionPlot)
export(moduleToModuleHeatmap)
export(name)
export(overlapComparisons)
export(performANOVA)
export(preservationComparisonPlot)
export(preservationComparisons)
export(runDME)
export(summarizeResults)
export(topNGenes)
import(SummarizedExperiment)
import(WGCNA)
import(dcanr)
import(dplyr)
import(flashClust)
import(ggalluvial)
import(ggplot2)
import(ggrepel)
import(methods)
import(patchwork)
import(readr)
import(stringr)
importFrom(cowplot,plot_grid)
importFrom(data.table,fwrite)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rgb)
importFrom(graphics,legend)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,degree)
importFrom(igraph,delete.edges)
importFrom(igraph,delete.vertices)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,plot.igraph)
importFrom(igraph,simplify)
importFrom(scales,rescale)
importFrom(stats,IQR)
importFrom(stats,anova)
importFrom(stats,dist)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,write.csv)
importFrom(utils,write.table)
