#' Permutation test results
#'
#' The results of running the PreservationPermutationTest in the astrocyte
#' vignette. This is provided since this function is quite slow. Please see
#' the astrocyte vignette for more details. 
#'
#' @docType data
#' @keywords datasets
#' @name permutationTestResults
#' @usage data(permutationTestResults)
#' @format A list of data.frames containing preservation results for each 
#' permutation
NULL